#include <xc.h>
#include <stdio.h>
#include "eeprom_M24128.h"


#define _I2C_ON_OFF(x)		SSP1CON1bits.SSPEN = (x)
#define _I2C_START		SSP1CON2bits.SEN = 1
#define _I2C_START_BIT		SSP1CON2bits.SEN
#define _I2C_RESTART		SSP1CON2bits.RSEN = 1
#define _I2C_RESTART_BIT	SSP1CON2bits.RSEN
#define _I2C_STOP		SSP1CON2bits.PEN = 1
#define _I2C_STOP_BIT		SSP1CON2bits.PEN
#define _I2C_RECEIVE		SSP1CON2bits.RCEN = 1
#define _I2C_RECEIVE_BIT	SSP1CON2bits.RCEN
#define _I2C_ACK_NACK(x)	SSP1CON2bits.ACKDT = (x)
#define _I2C_ACK_NACK_SEND	SSP1CON2bits.ACKEN = 1
#define _SSP1IF_BIT		PIR1bits.SSP1IF
#define _SSP1IF_BIT_SET(x)	PIR1bits.SSP1IF = (x)
#define _SSP1BUF(x)		SSP1BUF = (x)
#define _SSP1BUF_RD		SSP1BUF
#define _SLAVE_ACK_BIT		SSP1CON2bits.ACKSTAT
#define _I2C_CKP(x)		SSP1CON1bits.CKP = (x)

#define I2C_CTRL_BYTE_WR 	(0xA0) // 0b10100000
#define I2C_CTRL_BYTE_RD 	(0xA1) // 0b10100001

#define MAX_ADDRESS		(0x3FFF)
#define MAX_PAGE_WRITE_BYTE	(64)

#define EEPROM_DELAY_WRITE_MS	(7)

#define RC_SUCCESS 		0
#define RC_FAIL 		1
#define RC_I2C_ACK_ERROR 	2

#define ON			1
#define OFF			0
#define HIGH			1
#define LOW			0

#define _XTAL_FREQ 8000000

static void db_delay_ms(uint16_t ms);

static void db_delay_ms(uint16_t ms)
{
	while (ms > 0) 
	{
		__delay_ms(1);
		ms--;
	}
}

int eeprom_M24128_write(uint16_t word_address, uint8_t *u8_data)
{
	 // printf("EE-1\n\r");
	 // _SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt
	_I2C_ON_OFF(ON);		// enable the module
	
	_I2C_START;			// issue start condition
   	while (_I2C_START_BIT);		// wait for HW to clear it
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt

	_SSP1BUF(I2C_CTRL_BYTE_WR);	// Control Byte
	while (!_SSP1IF_BIT);		// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt

	// check for ACK (=0) from the Slave
	while (!_SLAVE_ACK_BIT);

	// printf("EE-2\n\r");
	
	// _SSP1BUF((uint8_t)(word_address >> 8));		// send MSB address
	_SSP1BUF(0x00);		// send MSB address
	while (!_SSP1IF_BIT);				// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);				// clear SSP2 interrupt
	
	// check for ACK (=0) from the Slave
	while (!_SLAVE_ACK_BIT);

	// printf("EE-3\n\r");
	
	// _SSP1BUF((uint8_t)(word_address & 0xFF));	// send LSB address
	_SSP1BUF(0x00);
	while (!_SSP1IF_BIT);				// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);				// clear SSP2 interrupt
	
	// check for ACK (=0) from the Slave
	while (!_SLAVE_ACK_BIT);

	// printf("EE-4\n\r");
	
	_SSP1BUF(0x12);				// load data
	while (!_SSP1IF_BIT);				// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);				// clear SSP2 interrupt
	
	// check for ACK (=0) from the Slave
	while (!_SLAVE_ACK_BIT);

	// printf("EE-5\n\r");
	
	_I2C_STOP;					// issue STOP condition
	while (_I2C_STOP_BIT);				// wait for HW to do it and clear its flag
	_I2C_ON_OFF(OFF);				// disable SSP2 module

	// mininum wait 5 ms
	db_delay_ms(EEPROM_DELAY_WRITE_MS);
	printf("EE-6\n\r");
	return RC_SUCCESS;
}

int eeprom_M24128_read(uint16_t word_address, uint8_t *u8_data)
{
	_I2C_ON_OFF(ON);		// enable the module
	_I2C_START;			// issue start condition
	while (_I2C_START_BIT);		// wait for HW to clear it
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt

	_SSP1BUF(I2C_CTRL_BYTE_WR);	// Control byte
	while (!_SSP1IF_BIT);		// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt
	// check for ACK (=0) from the Slave
	// db_delay_us\(10\);
	if (_SLAVE_ACK_BIT) 
	{
		_I2C_STOP;
		while (_I2C_STOP_BIT){};
		//printf("I2C error: %d\n\r", __LINE__);
		return RC_I2C_ACK_ERROR;
	}

	_SSP1BUF((uint8_t)(word_address >> 8));		// send MSB address
	while (!_SSP1IF_BIT);				// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);				// clear SSP2 interrupt
	// check for ACK (=0) from the Slave
	// db_delay_us\(10\);
	if (_SLAVE_ACK_BIT) 
	{
		_I2C_STOP;
		while (_I2C_STOP_BIT){};
		//printf("I2C error: %d\n\r", __LINE__);
		return RC_I2C_ACK_ERROR;
	}

	_SSP1BUF((uint8_t)(word_address & 0xFF));	// send LSB address
	while (!_SSP1IF_BIT);				// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);				// clear SSP2 interrupt
	// check for ACK (=0) from the Slave
	// db_delay_us\(10\);
	if (_SLAVE_ACK_BIT) 
	{
		_I2C_STOP;
		while (_I2C_STOP_BIT){};
		//printf("I2C error: %d\n\r", __LINE__);
		return RC_I2C_ACK_ERROR;
	}

	_I2C_RESTART;
	while (!_I2C_RESTART_BIT);	// wait for it to be done
	while (!_SSP1IF_BIT);		// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt

	_SSP1BUF(I2C_CTRL_BYTE_RD);	// Control byte
	while (!_SSP1IF_BIT);		// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt
	// check for ACK (=0) from the Slave
	// db_delay_us\(10\);
	if (_SLAVE_ACK_BIT) 
	{
		_I2C_STOP;
		while (_I2C_STOP_BIT){};
		//printf("I2C error: %d\n\r", __LINE__);
		return RC_I2C_ACK_ERROR;
	}

	_I2C_RECEIVE;
	while (_I2C_RECEIVE_BIT);
	while (!_SSP1IF_BIT);		// wait for it to be shifted and done
	*u8_data = _SSP1BUF_RD;
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt

	_I2C_ACK_NACK(HIGH);
	_I2C_ACK_NACK_SEND;

	while (!_SSP1IF_BIT);		// wait for it to be shifted and done
	_SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt

	_I2C_STOP;			// issue STOP condition
	while (_I2C_STOP_BIT);		// wait for HW to do it and clear its flag
	_I2C_ON_OFF(OFF);		// disable SSP2 module

	return RC_SUCCESS;
}
